#lang scheme

(require "setup.ss")

(provide new-midi-handle
         dispose-midi-handle
         with-midi-handle
         set-bank
         send-midi-command
         (struct-out midi-note-off)
         (struct-out midi-note-on)
         (struct-out midi-aftertouch))

;; based on http://recording.songstuff.com/articles.php?selected=55, it looks like midi messages are structured like this:

;; A midi message is three *bytes*.  I'm guessing the interface makes them three 32-bit values for performance reasons.
;; the first "status" byte must have a 1 in the hi-value bit.  The data-1 and data-2 bytes, whose meanings are determined by the 
;; status byte, have 0s as hi-value bits.

;; the following COPIED from http://recording.songstuff.com/articles.php?selected=55:
;; Message               Status Data 1          Data 2
;; ----------
;; Note Off              8n     Note Number       Velocity
;; Note On               9n     Note Number       Velocity
;; Polyphonic Aftertouch An     Note Number       Pressure
;; Control Change	 Bn     Controller Number Data
;; Program Change	 Cn     Program Number    Unused
;; Channel Aftertouch	 Dn     Pressure          Unused
;; Pitch Wheel           En     LSB               MSB


;; a midi-command is one of 
;; (make-midi-note-off channel vote velocity)
;; (make-midi-note-on channel note velocity)
(define-struct midi-note-off (channel pitch velocity))
(define-struct midi-note-on (channel pitch velocity))
(define-struct midi-aftertouch (channel pitch pressure))
;; ... add more as necessary.



(define kMidiMessage_ControlChange        #xB0)
(define kMidiMessage_BankMSBControl       #x00) ;; yucky hardware hack
(define kMidiMessage_ProgramChange        #xC0)
(define midi-command-on               #x90)
(define midi-command-off              #x80)
(define kMidiMessage_PolyphonicAftertouch #xA0)

(define (high-byte b)
  (arithmetic-shift b -8))

(define (low-byte b)
  (bitwise-and b #xff))

(define midiChannelInUse 0)

;; in a funny back-hack, one of the "control change" messages is used to enter the high 8 bits of the "program change" message.
;; set our bank (?)

;; set the "bank"; that is, choose a synthesizer voice:
(define (set-bank midi-handle banknum)
  ;; send the high byte of the bank number:
  (midi-command! midi-handle
                 (+ kMidiMessage_ControlChange midiChannelInUse)
                 kMidiMessage_BankMSBControl
                 (high-byte banknum))
  ;; send the low byte of the bank number:
  (midi-command! midi-handle
                 (+ kMidiMessage_ProgramChange midiChannelInUse)
                 (low-byte banknum)
                 0)) ; ignored, IIUC




;; send-midi-command : midi-command -> (void)
(define (send-midi-command handle command)
  (match command
    [(struct midi-note-on (channel pitch velocity)) (midi-command! handle (+ midi-command-on channel) pitch velocity)]
    [(struct midi-note-off (channel pitch velocity)) (midi-command! handle (+ midi-command-off channel) pitch velocity)]))


